-- create_exporter_position_table
CREATE TABLE EXPORTER_POSITION (
  PARTITION_ID numeric NOT NULL,
  EXPORTER VARCHAR(200) NULL,
  LAST_EXPORTED_POSITION BIGINT NULL,
  CREATED datetime NULL,
  LAST_UPDATED datetime NULL,
  CONSTRAINT PK_EXPORTER_POSITION PRIMARY KEY (PARTITION_ID)
);

-- create_process_deployment_table
CREATE TABLE PROCESS_DEFINITION (
  PROCESS_DEFINITION_KEY BIGINT NOT NULL,
  PROCESS_DEFINITION_ID VARCHAR(255) NULL,
  NAME VARCHAR(255) NULL,
  RESOURCE_NAME VARCHAR(4000) NULL,
  BPMN_XML LONGTEXT NULL,
  TENANT_ID VARCHAR(255) NULL,
  VERSION SMALLINT NULL,
  VERSION_TAG VARCHAR(255) NULL,
  FORM_ID VARCHAR(255) NULL,
  CONSTRAINT PK_PROCESS_DEFINITION PRIMARY KEY (PROCESS_DEFINITION_KEY)
);

CREATE INDEX IDX_PROCESS_DEFINITION_ID ON PROCESS_DEFINITION(PROCESS_DEFINITION_ID);

-- create_process_instance_table
CREATE TABLE PROCESS_INSTANCE (
  PROCESS_INSTANCE_KEY BIGINT NOT NULL,
  PROCESS_DEFINITION_ID VARCHAR(200) NULL,
  PROCESS_DEFINITION_KEY BIGINT NULL,
  STATE VARCHAR(20) NULL,
  START_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  END_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  TENANT_ID VARCHAR(255) NULL,
  PARENT_PROCESS_INSTANCE_KEY BIGINT NULL,
  PARENT_ELEMENT_INSTANCE_KEY BIGINT NULL,
  NUM_INCIDENTS SMALLINT NULL,
  ELEMENT_ID VARCHAR(255) NULL,
  VERSION SMALLINT NULL,
  PARTITION_ID numeric NULL,
  HISTORY_CLEANUP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  CONSTRAINT PK_PROCESS_INSTANCE PRIMARY KEY (PROCESS_INSTANCE_KEY)
);

-- create_variable_table
CREATE TABLE VARIABLE (
  VAR_KEY BIGINT NOT NULL,
  PROCESS_INSTANCE_KEY BIGINT NULL,
  PROCESS_DEFINITION_ID VARCHAR(255) NULL,
  SCOPE_KEY BIGINT NULL,
  TYPE VARCHAR(255) NULL,
  VAR_NAME VARCHAR(255) NULL,
  DOUBLE_VALUE DOUBLE NULL,
  LONG_VALUE BIGINT NULL,
  VAR_VALUE VARCHAR(8192) NULL,
  VAR_FULL_VALUE LONGTEXT NULL,
  TENANT_ID VARCHAR(255) NULL,
  IS_PREVIEW TINYINT NULL,
  PARTITION_ID numeric NULL,
  HISTORY_CLEANUP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  CONSTRAINT PK_VARIABLE PRIMARY KEY (VAR_KEY)
);

CREATE INDEX IDX_VARIABLE_PROCESS_INSTANCE_KEY ON VARIABLE(PROCESS_INSTANCE_KEY);

CREATE INDEX IDX_VARIABLE_TENANT_ID ON VARIABLE(TENANT_ID);

-- create_flow_node_table
CREATE TABLE FLOW_NODE_INSTANCE (
  FLOW_NODE_INSTANCE_KEY BIGINT NOT NULL,
  FLOW_NODE_ID VARCHAR(255) NULL,
  PROCESS_INSTANCE_KEY BIGINT NULL,
  PROCESS_DEFINITION_ID VARCHAR(255) NULL,
  PROCESS_DEFINITION_KEY BIGINT NULL,
  TYPE VARCHAR(100) NULL,
  STATE VARCHAR(20) NULL,
  START_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  END_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  TREE_PATH VARCHAR(4000) NULL,
  TENANT_ID VARCHAR(255) NULL,
  INCIDENT_KEY BIGINT NULL,
  NUM_SUBPROCESS_INCIDENTS SMALLINT NULL,
  PARTITION_ID numeric NULL,
  HISTORY_CLEANUP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  CONSTRAINT PK_FLOW_NODE_INSTANCE PRIMARY KEY (FLOW_NODE_INSTANCE_KEY)
);

-- create_usertask_table
CREATE TABLE USER_TASK (
  USER_TASK_KEY BIGINT NOT NULL,
  ELEMENT_ID VARCHAR(255) NULL,
  PROCESS_DEFINITION_ID VARCHAR(255) NULL,
  CREATION_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  COMPLETION_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  ASSIGNEE VARCHAR(255) NULL,
  STATE VARCHAR(20) NULL,
  FORM_KEY BIGINT NULL,
  PROCESS_DEFINITION_KEY BIGINT NULL,
  PROCESS_INSTANCE_KEY BIGINT NULL,
  ELEMENT_INSTANCE_KEY BIGINT NULL,
  TENANT_ID VARCHAR(255) NULL,
  DUE_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  FOLLOW_UP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  EXTERNAL_FORM_REFERENCE VARCHAR(255) NULL,
  PROCESS_DEFINITION_VERSION SMALLINT NULL,
  CUSTOM_HEADERS LONGTEXT NULL,
  PRIORITY INT NULL,
  PARTITION_ID numeric NULL,
  HISTORY_CLEANUP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  CONSTRAINT PK_USER_TASK PRIMARY KEY (USER_TASK_KEY)
);

CREATE TABLE CANDIDATE_USER (
  USER_TASK_KEY BIGINT NULL,
  CANDIDATE_USER VARCHAR(255) NULL,
  CONSTRAINT FK_CANDIDATE_USER_USER_TASK FOREIGN KEY (USER_TASK_KEY) REFERENCES USER_TASK(USER_TASK_KEY) ON DELETE CASCADE
);

CREATE TABLE CANDIDATE_GROUP (
  USER_TASK_KEY BIGINT NULL,
  CANDIDATE_GROUP VARCHAR(255) NULL,
  CONSTRAINT FK_CANDIDATE_GROUP_USER_TASK FOREIGN KEY (USER_TASK_KEY) REFERENCES USER_TASK(USER_TASK_KEY) ON DELETE CASCADE
);

CREATE INDEX IDX_CANDIDATE_USER_USER_TASK_KEY ON CANDIDATE_USER(USER_TASK_KEY);

CREATE INDEX IDX_CANDIDATE_GROUP_USER_TASK_KEY ON CANDIDATE_GROUP(USER_TASK_KEY);

-- create_incident
CREATE TABLE INCIDENT (
  INCIDENT_KEY BIGINT NOT NULL,
  FLOW_NODE_INSTANCE_KEY BIGINT NULL,
  FLOW_NODE_ID VARCHAR(255) NULL,
  PROCESS_INSTANCE_KEY BIGINT NULL,
  PROCESS_DEFINITION_ID VARCHAR(255) NULL,
  PROCESS_DEFINITION_KEY BIGINT NULL,
  ERROR_MESSAGE VARCHAR(4000) NULL,
  ERROR_TYPE VARCHAR(100) NULL,
  STATE VARCHAR(20) NULL,
  CREATION_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  END_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  TREE_PATH VARCHAR(4000) NULL,
  TENANT_ID VARCHAR(255) NULL,
  JOB_KEY BIGINT NULL,
  SCOPE_KEY BIGINT NULL,
  PARTITION_ID numeric NULL,
  HISTORY_CLEANUP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  CONSTRAINT PK_INCIDENT PRIMARY KEY (INCIDENT_KEY)
);

-- create_decision_definition_table
CREATE TABLE DECISION_DEFINITION (
  DECISION_DEFINITION_KEY BIGINT NOT NULL,
  DECISION_DEFINITION_ID VARCHAR(255) NULL,
  NAME VARCHAR(255) NULL,
  TENANT_ID VARCHAR(255) NULL,
  VERSION SMALLINT NULL,
  DECISION_REQUIREMENTS_KEY BIGINT NULL,
  DECISION_REQUIREMENTS_ID VARCHAR(255) NULL,
  CONSTRAINT PK_DECISION_DEFINITION PRIMARY KEY (DECISION_DEFINITION_KEY)
);

CREATE INDEX IDX_DECISION_DEFINITION_ID ON DECISION_DEFINITION(DECISION_DEFINITION_ID);

-- create_decision_requirements_table
CREATE TABLE DECISION_REQUIREMENTS (
  DECISION_REQUIREMENTS_KEY BIGINT NOT NULL,
  DECISION_REQUIREMENTS_ID VARCHAR(255) NULL,
  NAME VARCHAR(255) NULL,
  TENANT_ID VARCHAR(255) NULL,
  VERSION SMALLINT NULL,
  RESOURCE_NAME VARCHAR(4000) NULL,
  XML LONGTEXT NULL,
  CONSTRAINT PK_DECISION_REQUIREMENTS PRIMARY KEY (DECISION_REQUIREMENTS_KEY)
);

CREATE INDEX IDX_DECISION_REQUIREMENTS_ID ON DECISION_REQUIREMENTS(DECISION_REQUIREMENTS_ID);

-- create_decision_instance_table
CREATE TABLE DECISION_INSTANCE (
  DECISION_INSTANCE_ID VARCHAR(255) NOT NULL,
  DECISION_INSTANCE_KEY BIGINT NULL,
  PROCESS_INSTANCE_KEY BIGINT NULL,
  PROCESS_DEFINITION_ID VARCHAR(255) NULL,
  PROCESS_DEFINITION_KEY BIGINT NULL,
  DECISION_DEFINITION_ID VARCHAR(255) NULL,
  DECISION_DEFINITION_KEY BIGINT NULL,
  DECISION_REQUIREMENTS_ID VARCHAR(255) NULL,
  DECISION_REQUIREMENTS_KEY BIGINT NULL,
  FLOW_NODE_ID VARCHAR(255) NULL,
  FLOW_NODE_INSTANCE_KEY BIGINT NULL,
  ROOT_DECISION_DEFINITION_KEY BIGINT NULL,
  EVALUATION_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  TYPE VARCHAR(255) NULL,
  STATE VARCHAR(255) NULL,
  RESULT VARCHAR(255) NULL,
  EVALUATION_FAILURE VARCHAR(255) NULL,
  TENANT_ID VARCHAR(255) NULL,
  PARTITION_ID numeric NULL,
  HISTORY_CLEANUP_DATE TIMESTAMP WITH TIME ZONE(3) NULL,
  CONSTRAINT PK_DECISION_INSTANCE PRIMARY KEY (DECISION_INSTANCE_ID)
);

-- create_decision_instance_input_table
CREATE TABLE DECISION_INSTANCE_INPUT (
  DECISION_INSTANCE_ID VARCHAR(255) NOT NULL,
  INPUT_ID VARCHAR(255) NOT NULL,
  INPUT_NAME VARCHAR(255) NULL,
  INPUT_VALUE VARCHAR(4000) NULL,
  CONSTRAINT PK_DECISION_INSTANCE_INPUT PRIMARY KEY (DECISION_INSTANCE_ID, INPUT_ID),
  CONSTRAINT FK_DEC_INST_INPUT_DEC_INST FOREIGN KEY (DECISION_INSTANCE_ID) REFERENCES DECISION_INSTANCE(DECISION_INSTANCE_ID) ON DELETE CASCADE
);

-- create_decision_instance_output_table
CREATE TABLE DECISION_INSTANCE_OUTPUT (
  DECISION_INSTANCE_ID VARCHAR(255) NOT NULL,
  OUTPUT_ID VARCHAR(255) NOT NULL,
  RULE_INDEX SMALLINT NOT NULL,
  OUTPUT_NAME VARCHAR(255) NULL,
  OUTPUT_VALUE VARCHAR(4000) NULL,
  RULE_ID VARCHAR(255) NULL,
  CONSTRAINT PK_DECISION_INSTANCE_OUTPUT PRIMARY KEY (DECISION_INSTANCE_ID, OUTPUT_ID, RULE_INDEX),
  CONSTRAINT FK_DEC_INST_OUTPUT_DEC_INST FOREIGN KEY (DECISION_INSTANCE_ID) REFERENCES DECISION_INSTANCE(DECISION_INSTANCE_ID) ON DELETE CASCADE
);

-- create_user_table
CREATE TABLE USERS (
  USERNAME VARCHAR(255) NOT NULL,
  USER_KEY BIGINT NULL,
  NAME VARCHAR(255) NULL,
  EMAIL VARCHAR(255) NULL,
  PASSWORD VARCHAR(255) NULL,
  CONSTRAINT PK_USERS PRIMARY KEY (USERNAME)
);

-- create_form_table
CREATE TABLE FORM (
  FORM_KEY BIGINT NOT NULL,
  FORM_ID VARCHAR(255) NULL,
  TENANT_ID VARCHAR(255) NULL,
  ` SCHEMA ` LONGTEXT NULL,
  VERSION BIGINT NULL,
  IS_DELETED TINYINT NULL,
  CONSTRAINT PK_FORM PRIMARY KEY (FORM_KEY)
);

CREATE INDEX IDX_FORM_ID ON FORM(FORM_ID);

CREATE INDEX IDX_FORM_TENANT_ID ON FORM(TENANT_ID);

-- create_mapping_table
CREATE TABLE MAPPINGS (
  MAPPING_KEY BIGINT NOT NULL,
  CLAIM_NAME VARCHAR(255) NULL,
  CLAIM_VALUE VARCHAR(255) NULL,
  NAME VARCHAR(255) NULL,
  CONSTRAINT PK_MAPPINGS PRIMARY KEY (MAPPING_KEY)
);

-- create_tenant_table
CREATE TABLE TENANT (
  TENANT_ID VARCHAR(255) NOT NULL,
  TENANT_KEY BIGINT NULL,
  NAME VARCHAR(255) NULL,
  ` DESCRIPTION ` VARCHAR(255) NULL,
  CONSTRAINT PK_TENANT PRIMARY KEY (TENANT_ID)
);

-- tenant_members_table
CREATE TABLE TENANT_MEMBER (
  TENANT_ID VARCHAR(255) NOT NULL,
  ENTITY_ID VARCHAR(255) NOT NULL,
  ENTITY_TYPE VARCHAR(255) NULL,
  CONSTRAINT PK_TENANT_MEMBER PRIMARY KEY (TENANT_ID, ENTITY_ID)
);

-- create_role_table
CREATE TABLE ROLES (
  ROLE_KEY BIGINT NOT NULL,
  NAME VARCHAR(255) NULL,
  CONSTRAINT PK_ROLES PRIMARY KEY (ROLE_KEY)
);

-- role_members_table
CREATE TABLE ROLE_MEMBER (
  ROLE_KEY BIGINT NOT NULL,
  ENTITY_KEY BIGINT NOT NULL,
  ENTITY_TYPE VARCHAR(255) NULL,
  CONSTRAINT PK_ROLE_MEMBER PRIMARY KEY (ROLE_KEY, ENTITY_KEY)
);

-- create_group_table
CREATE TABLE ` GROUPS ` (
  GROUP_KEY BIGINT NOT NULL,
  NAME VARCHAR(255) NULL,
  CONSTRAINT PK_GROUPS PRIMARY KEY (GROUP_KEY)
);

-- group_members_table
CREATE TABLE GROUP_MEMBER (
  GROUP_KEY BIGINT NOT NULL,
  ENTITY_KEY BIGINT NOT NULL,
  ENTITY_TYPE VARCHAR(255) NULL,
  CONSTRAINT PK_GROUP_MEMBER PRIMARY KEY (GROUP_KEY, ENTITY_KEY)
);

-- create_authorization_permission_table
CREATE TABLE AUTHORIZATIONS (
  AUTHORIZATION_KEY BIGINT NULL,
  OWNER_ID VARCHAR(255) NULL,
  OWNER_TYPE VARCHAR(255) NULL,
  RESOURCE_TYPE VARCHAR(255) NULL,
  RESOURCE_ID VARCHAR(255) NULL,
  PERMISSION_TYPE VARCHAR(255) NULL
);

CREATE INDEX IDX_AUTHORIZATIONS_AUTHORIZATION_KEY ON AUTHORIZATIONS(AUTHORIZATION_KEY);

CREATE INDEX IDX_AUTHORIZATIONS ON AUTHORIZATIONS(OWNER_ID, OWNER_TYPE, RESOURCE_TYPE);

