-- create_exporter_position_table
CREATE TABLE EXPORTER_POSITION (
  PARTITION_ID NUMBER NOT NULL,
  EXPORTER VARCHAR2(200),
  LAST_EXPORTED_POSITION NUMBER(38, 0),
  CREATED TIMESTAMP,
  LAST_UPDATED TIMESTAMP,
  CONSTRAINT PK_EXPORTER_POSITION PRIMARY KEY (PARTITION_ID)
);

-- create_process_deployment_table
CREATE TABLE PROCESS_DEFINITION (
  PROCESS_DEFINITION_KEY NUMBER(38, 0) NOT NULL,
  PROCESS_DEFINITION_ID VARCHAR2(255),
  NAME VARCHAR2(255),
  RESOURCE_NAME VARCHAR2(4000),
  BPMN_XML CLOB,
  TENANT_ID VARCHAR2(255),
  VERSION NUMBER(5),
  VERSION_TAG VARCHAR2(255),
  FORM_ID VARCHAR2(255),
  CONSTRAINT PK_PROCESS_DEFINITION PRIMARY KEY (PROCESS_DEFINITION_KEY)
);

CREATE INDEX IDX_PROCESS_DEFINITION_ID ON PROCESS_DEFINITION(PROCESS_DEFINITION_ID);

-- create_process_instance_table
CREATE TABLE PROCESS_INSTANCE (
  PROCESS_INSTANCE_KEY NUMBER(38, 0) NOT NULL,
  PROCESS_DEFINITION_ID VARCHAR2(200),
  PROCESS_DEFINITION_KEY NUMBER(38, 0),
  STATE VARCHAR2(20),
  START_DATE TIMESTAMP(3) WITH TIME ZONE,
  END_DATE TIMESTAMP(3) WITH TIME ZONE,
  TENANT_ID VARCHAR2(255),
  PARENT_PROCESS_INSTANCE_KEY NUMBER(38, 0),
  PARENT_ELEMENT_INSTANCE_KEY NUMBER(38, 0),
  NUM_INCIDENTS NUMBER(5),
  ELEMENT_ID VARCHAR2(255),
  VERSION NUMBER(5),
  PARTITION_ID NUMBER,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT PK_PROCESS_INSTANCE PRIMARY KEY (PROCESS_INSTANCE_KEY)
);

-- create_variable_table
CREATE TABLE VARIABLE (
  VAR_KEY NUMBER(38, 0) NOT NULL,
  PROCESS_INSTANCE_KEY NUMBER(38, 0),
  PROCESS_DEFINITION_ID VARCHAR2(255),
  SCOPE_KEY NUMBER(38, 0),
  TYPE VARCHAR2(255),
  VAR_NAME VARCHAR2(255),
  DOUBLE_VALUE DOUBLE PRECISION,
  LONG_VALUE NUMBER(38, 0),
  VAR_VALUE VARCHAR2(8192),
  VAR_FULL_VALUE CLOB,
  TENANT_ID VARCHAR2(255),
  IS_PREVIEW BOOLEAN,
  PARTITION_ID NUMBER,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT PK_VARIABLE PRIMARY KEY (VAR_KEY)
);

CREATE INDEX IDX_VARIABLE_PROCESS_INSTANCE_KEY ON VARIABLE(PROCESS_INSTANCE_KEY);

CREATE INDEX IDX_VARIABLE_TENANT_ID ON VARIABLE(TENANT_ID);

-- create_flow_node_table
CREATE TABLE FLOW_NODE_INSTANCE (
  FLOW_NODE_INSTANCE_KEY NUMBER(38, 0) NOT NULL,
  FLOW_NODE_ID VARCHAR2(255),
  PROCESS_INSTANCE_KEY NUMBER(38, 0),
  PROCESS_DEFINITION_ID VARCHAR2(255),
  PROCESS_DEFINITION_KEY NUMBER(38, 0),
  TYPE VARCHAR2(100),
  STATE VARCHAR2(20),
  START_DATE TIMESTAMP(3) WITH TIME ZONE,
  END_DATE TIMESTAMP(3) WITH TIME ZONE,
  TREE_PATH VARCHAR2(4000),
  TENANT_ID VARCHAR2(255),
  INCIDENT_KEY NUMBER(38, 0),
  NUM_SUBPROCESS_INCIDENTS NUMBER(5),
  PARTITION_ID NUMBER,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT PK_FLOW_NODE_INSTANCE PRIMARY KEY (FLOW_NODE_INSTANCE_KEY)
);

-- create_usertask_table
CREATE TABLE USER_TASK (
  USER_TASK_KEY NUMBER(38, 0) NOT NULL,
  ELEMENT_ID VARCHAR2(255),
  PROCESS_DEFINITION_ID VARCHAR2(255),
  CREATION_DATE TIMESTAMP(3) WITH TIME ZONE,
  COMPLETION_DATE TIMESTAMP(3) WITH TIME ZONE,
  ASSIGNEE VARCHAR2(255),
  STATE VARCHAR2(20),
  FORM_KEY NUMBER(38, 0),
  PROCESS_DEFINITION_KEY NUMBER(38, 0),
  PROCESS_INSTANCE_KEY NUMBER(38, 0),
  ELEMENT_INSTANCE_KEY NUMBER(38, 0),
  TENANT_ID VARCHAR2(255),
  DUE_DATE TIMESTAMP(3) WITH TIME ZONE,
  FOLLOW_UP_DATE TIMESTAMP(3) WITH TIME ZONE,
  EXTERNAL_FORM_REFERENCE VARCHAR2(255),
  PROCESS_DEFINITION_VERSION NUMBER(5),
  CUSTOM_HEADERS CLOB,
  PRIORITY INTEGER,
  PARTITION_ID NUMBER,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT PK_USER_TASK PRIMARY KEY (USER_TASK_KEY)
);

CREATE TABLE CANDIDATE_USER (
  USER_TASK_KEY NUMBER(38, 0),
  CANDIDATE_USER VARCHAR2(255),
  CONSTRAINT FK_CANDIDATE_USER_USER_TASK FOREIGN KEY (USER_TASK_KEY) REFERENCES USER_TASK(USER_TASK_KEY) ON DELETE CASCADE
);

CREATE TABLE CANDIDATE_GROUP (
  USER_TASK_KEY NUMBER(38, 0),
  CANDIDATE_GROUP VARCHAR2(255),
  CONSTRAINT FK_CANDIDATE_GROUP_USER_TASK FOREIGN KEY (USER_TASK_KEY) REFERENCES USER_TASK(USER_TASK_KEY) ON DELETE CASCADE
);

CREATE INDEX IDX_CANDIDATE_USER_USER_TASK_KEY ON CANDIDATE_USER(USER_TASK_KEY);

CREATE INDEX IDX_CANDIDATE_GROUP_USER_TASK_KEY ON CANDIDATE_GROUP(USER_TASK_KEY);

-- create_incident
CREATE TABLE INCIDENT (
  INCIDENT_KEY NUMBER(38, 0) NOT NULL,
  FLOW_NODE_INSTANCE_KEY NUMBER(38, 0),
  FLOW_NODE_ID VARCHAR2(255),
  PROCESS_INSTANCE_KEY NUMBER(38, 0),
  PROCESS_DEFINITION_ID VARCHAR2(255),
  PROCESS_DEFINITION_KEY NUMBER(38, 0),
  ERROR_MESSAGE VARCHAR2(4000),
  ERROR_TYPE VARCHAR2(100),
  STATE VARCHAR2(20),
  CREATION_DATE TIMESTAMP(3) WITH TIME ZONE,
  END_DATE TIMESTAMP(3) WITH TIME ZONE,
  TREE_PATH VARCHAR2(4000),
  TENANT_ID VARCHAR2(255),
  JOB_KEY NUMBER(38, 0),
  SCOPE_KEY NUMBER(38, 0),
  PARTITION_ID NUMBER,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT PK_INCIDENT PRIMARY KEY (INCIDENT_KEY)
);

-- create_decision_definition_table
CREATE TABLE DECISION_DEFINITION (
  DECISION_DEFINITION_KEY NUMBER(38, 0) NOT NULL,
  DECISION_DEFINITION_ID VARCHAR2(255),
  NAME VARCHAR2(255),
  TENANT_ID VARCHAR2(255),
  VERSION NUMBER(5),
  DECISION_REQUIREMENTS_KEY NUMBER(38, 0),
  DECISION_REQUIREMENTS_ID VARCHAR2(255),
  CONSTRAINT PK_DECISION_DEFINITION PRIMARY KEY (DECISION_DEFINITION_KEY)
);

CREATE INDEX IDX_DECISION_DEFINITION_ID ON DECISION_DEFINITION(DECISION_DEFINITION_ID);

-- create_decision_requirements_table
CREATE TABLE DECISION_REQUIREMENTS (
  DECISION_REQUIREMENTS_KEY NUMBER(38, 0) NOT NULL,
  DECISION_REQUIREMENTS_ID VARCHAR2(255),
  NAME VARCHAR2(255),
  TENANT_ID VARCHAR2(255),
  VERSION NUMBER(5),
  RESOURCE_NAME VARCHAR2(4000),
  XML CLOB,
  CONSTRAINT PK_DECISION_REQUIREMENTS PRIMARY KEY (DECISION_REQUIREMENTS_KEY)
);

CREATE INDEX IDX_DECISION_REQUIREMENTS_ID ON DECISION_REQUIREMENTS(DECISION_REQUIREMENTS_ID);

-- create_decision_instance_table
CREATE TABLE DECISION_INSTANCE (
  DECISION_INSTANCE_ID VARCHAR2(255) NOT NULL,
  DECISION_INSTANCE_KEY NUMBER(38, 0),
  PROCESS_INSTANCE_KEY NUMBER(38, 0),
  PROCESS_DEFINITION_ID VARCHAR2(255),
  PROCESS_DEFINITION_KEY NUMBER(38, 0),
  DECISION_DEFINITION_ID VARCHAR2(255),
  DECISION_DEFINITION_KEY NUMBER(38, 0),
  DECISION_REQUIREMENTS_ID VARCHAR2(255),
  DECISION_REQUIREMENTS_KEY NUMBER(38, 0),
  FLOW_NODE_ID VARCHAR2(255),
  FLOW_NODE_INSTANCE_KEY NUMBER(38, 0),
  ROOT_DECISION_DEFINITION_KEY NUMBER(38, 0),
  EVALUATION_DATE TIMESTAMP(3) WITH TIME ZONE,
  TYPE VARCHAR2(255),
  STATE VARCHAR2(255),
  RESULT VARCHAR2(255),
  EVALUATION_FAILURE VARCHAR2(255),
  TENANT_ID VARCHAR2(255),
  PARTITION_ID NUMBER,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT PK_DECISION_INSTANCE PRIMARY KEY (DECISION_INSTANCE_ID)
);

-- create_decision_instance_input_table
CREATE TABLE DECISION_INSTANCE_INPUT (
  DECISION_INSTANCE_ID VARCHAR2(255) NOT NULL,
  INPUT_ID VARCHAR2(255) NOT NULL,
  INPUT_NAME VARCHAR2(255),
  INPUT_VALUE VARCHAR2(4000),
  CONSTRAINT PK_DECISION_INSTANCE_INPUT PRIMARY KEY (DECISION_INSTANCE_ID, INPUT_ID),
  CONSTRAINT FK_DEC_INST_INPUT_DEC_INST FOREIGN KEY (DECISION_INSTANCE_ID) REFERENCES DECISION_INSTANCE(DECISION_INSTANCE_ID) ON DELETE CASCADE
);

-- create_decision_instance_output_table
CREATE TABLE DECISION_INSTANCE_OUTPUT (
  DECISION_INSTANCE_ID VARCHAR2(255) NOT NULL,
  OUTPUT_ID VARCHAR2(255) NOT NULL,
  RULE_INDEX NUMBER(5) NOT NULL,
  OUTPUT_NAME VARCHAR2(255),
  OUTPUT_VALUE VARCHAR2(4000),
  RULE_ID VARCHAR2(255),
  CONSTRAINT PK_DECISION_INSTANCE_OUTPUT PRIMARY KEY (DECISION_INSTANCE_ID, OUTPUT_ID, RULE_INDEX),
  CONSTRAINT FK_DEC_INST_OUTPUT_DEC_INST FOREIGN KEY (DECISION_INSTANCE_ID) REFERENCES DECISION_INSTANCE(DECISION_INSTANCE_ID) ON DELETE CASCADE
);

-- create_user_table
CREATE TABLE USERS (
  USERNAME VARCHAR2(255) NOT NULL,
  USER_KEY NUMBER(38, 0),
  NAME VARCHAR2(255),
  EMAIL VARCHAR2(255),
  PASSWORD VARCHAR2(255),
  CONSTRAINT PK_USERS PRIMARY KEY (USERNAME)
);

-- create_form_table
CREATE TABLE FORM (
  FORM_KEY NUMBER(38, 0) NOT NULL,
  FORM_ID VARCHAR2(255),
  TENANT_ID VARCHAR2(255),
  SCHEMA CLOB,
  VERSION NUMBER(38, 0),
  IS_DELETED BOOLEAN,
  CONSTRAINT PK_FORM PRIMARY KEY (FORM_KEY)
);

CREATE INDEX IDX_FORM_ID ON FORM(FORM_ID);

CREATE INDEX IDX_FORM_TENANT_ID ON FORM(TENANT_ID);

-- create_mapping_table
CREATE TABLE MAPPINGS (
  MAPPING_KEY NUMBER(38, 0) NOT NULL,
  CLAIM_NAME VARCHAR2(255),
  CLAIM_VALUE VARCHAR2(255),
  NAME VARCHAR2(255),
  CONSTRAINT PK_MAPPINGS PRIMARY KEY (MAPPING_KEY)
);

-- create_tenant_table
CREATE TABLE TENANT (
  TENANT_ID VARCHAR2(255) NOT NULL,
  TENANT_KEY NUMBER(38, 0),
  NAME VARCHAR2(255),
  DESCRIPTION VARCHAR2(255),
  CONSTRAINT PK_TENANT PRIMARY KEY (TENANT_ID)
);

-- tenant_members_table
CREATE TABLE TENANT_MEMBER (
  TENANT_ID VARCHAR2(255) NOT NULL,
  ENTITY_ID VARCHAR2(255) NOT NULL,
  ENTITY_TYPE VARCHAR2(255),
  CONSTRAINT PK_TENANT_MEMBER PRIMARY KEY (TENANT_ID, ENTITY_ID)
);

-- create_role_table
CREATE TABLE ROLES (
  ROLE_KEY NUMBER(38, 0) NOT NULL,
  NAME VARCHAR2(255),
  CONSTRAINT PK_ROLES PRIMARY KEY (ROLE_KEY)
);

-- role_members_table
CREATE TABLE ROLE_MEMBER (
  ROLE_KEY NUMBER(38, 0) NOT NULL,
  ENTITY_KEY NUMBER(38, 0) NOT NULL,
  ENTITY_TYPE VARCHAR2(255),
  CONSTRAINT PK_ROLE_MEMBER PRIMARY KEY (ROLE_KEY, ENTITY_KEY)
);

-- create_group_table
CREATE TABLE GROUPS (
  GROUP_KEY NUMBER(38, 0) NOT NULL,
  NAME VARCHAR2(255),
  CONSTRAINT PK_GROUPS PRIMARY KEY (GROUP_KEY)
);

-- group_members_table
CREATE TABLE GROUP_MEMBER (
  GROUP_KEY NUMBER(38, 0) NOT NULL,
  ENTITY_KEY NUMBER(38, 0) NOT NULL,
  ENTITY_TYPE VARCHAR2(255),
  CONSTRAINT PK_GROUP_MEMBER PRIMARY KEY (GROUP_KEY, ENTITY_KEY)
);

-- create_authorization_permission_table
CREATE TABLE AUTHORIZATIONS (
  AUTHORIZATION_KEY NUMBER(38, 0),
  OWNER_ID VARCHAR2(255),
  OWNER_TYPE VARCHAR2(255),
  RESOURCE_TYPE VARCHAR2(255),
  RESOURCE_ID VARCHAR2(255),
  PERMISSION_TYPE VARCHAR2(255)
);

CREATE INDEX IDX_AUTHORIZATIONS_AUTHORIZATION_KEY ON AUTHORIZATIONS(AUTHORIZATION_KEY);

CREATE INDEX IDX_AUTHORIZATIONS ON AUTHORIZATIONS(OWNER_ID, OWNER_TYPE, RESOURCE_TYPE);

