-- create_exporter_position_table
CREATE TABLE EXPORTER_POSITION (
  PARTITION_ID numeric NOT NULL,
  EXPORTER VARCHAR(200),
  LAST_EXPORTED_POSITION BIGINT,
  CREATED TIMESTAMP WITHOUT TIME ZONE,
  LAST_UPDATED TIMESTAMP WITHOUT TIME ZONE,
  CONSTRAINT "EXPORTER_POSITION_pkey" PRIMARY KEY (PARTITION_ID)
);

-- create_process_deployment_table
CREATE TABLE PROCESS_DEFINITION (
  PROCESS_DEFINITION_KEY BIGINT NOT NULL,
  PROCESS_DEFINITION_ID VARCHAR(255),
  NAME VARCHAR(255),
  RESOURCE_NAME VARCHAR(4000),
  BPMN_XML TEXT,
  TENANT_ID VARCHAR(255),
  VERSION SMALLINT,
  VERSION_TAG VARCHAR(255),
  FORM_ID VARCHAR(255),
  CONSTRAINT "PROCESS_DEFINITION_pkey" PRIMARY KEY (PROCESS_DEFINITION_KEY)
);

CREATE INDEX IDX_PROCESS_DEFINITION_ID ON PROCESS_DEFINITION(PROCESS_DEFINITION_ID);

-- create_process_instance_table
CREATE TABLE PROCESS_INSTANCE (
  PROCESS_INSTANCE_KEY BIGINT NOT NULL,
  PROCESS_DEFINITION_ID VARCHAR(200),
  PROCESS_DEFINITION_KEY BIGINT,
  STATE VARCHAR(20),
  START_DATE TIMESTAMP(3) WITH TIME ZONE,
  END_DATE TIMESTAMP(3) WITH TIME ZONE,
  TENANT_ID VARCHAR(255),
  PARENT_PROCESS_INSTANCE_KEY BIGINT,
  PARENT_ELEMENT_INSTANCE_KEY BIGINT,
  NUM_INCIDENTS SMALLINT,
  ELEMENT_ID VARCHAR(255),
  VERSION SMALLINT,
  PARTITION_ID numeric,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT "PROCESS_INSTANCE_pkey" PRIMARY KEY (PROCESS_INSTANCE_KEY)
);

-- create_variable_table
CREATE TABLE VARIABLE (
  VAR_KEY BIGINT NOT NULL,
  PROCESS_INSTANCE_KEY BIGINT,
  PROCESS_DEFINITION_ID VARCHAR(255),
  SCOPE_KEY BIGINT,
  TYPE VARCHAR(255),
  VAR_NAME VARCHAR(255),
  DOUBLE_VALUE DOUBLE PRECISION,
  LONG_VALUE BIGINT,
  VAR_VALUE VARCHAR(8192),
  VAR_FULL_VALUE TEXT,
  TENANT_ID VARCHAR(255),
  IS_PREVIEW BOOLEAN,
  PARTITION_ID numeric,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT "VARIABLE_pkey" PRIMARY KEY (VAR_KEY)
);

CREATE INDEX IDX_VARIABLE_PROCESS_INSTANCE_KEY ON VARIABLE(PROCESS_INSTANCE_KEY);

CREATE INDEX IDX_VARIABLE_TENANT_ID ON VARIABLE(TENANT_ID);

-- create_flow_node_table
CREATE TABLE FLOW_NODE_INSTANCE (
  FLOW_NODE_INSTANCE_KEY BIGINT NOT NULL,
  FLOW_NODE_ID VARCHAR(255),
  PROCESS_INSTANCE_KEY BIGINT,
  PROCESS_DEFINITION_ID VARCHAR(255),
  PROCESS_DEFINITION_KEY BIGINT,
  TYPE VARCHAR(100),
  STATE VARCHAR(20),
  START_DATE TIMESTAMP(3) WITH TIME ZONE,
  END_DATE TIMESTAMP(3) WITH TIME ZONE,
  TREE_PATH VARCHAR(4000),
  TENANT_ID VARCHAR(255),
  INCIDENT_KEY BIGINT,
  NUM_SUBPROCESS_INCIDENTS SMALLINT,
  PARTITION_ID numeric,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT "FLOW_NODE_INSTANCE_pkey" PRIMARY KEY (FLOW_NODE_INSTANCE_KEY)
);

-- create_usertask_table
CREATE TABLE USER_TASK (
  USER_TASK_KEY BIGINT NOT NULL,
  ELEMENT_ID VARCHAR(255),
  PROCESS_DEFINITION_ID VARCHAR(255),
  CREATION_DATE TIMESTAMP(3) WITH TIME ZONE,
  COMPLETION_DATE TIMESTAMP(3) WITH TIME ZONE,
  ASSIGNEE VARCHAR(255),
  STATE VARCHAR(20),
  FORM_KEY BIGINT,
  PROCESS_DEFINITION_KEY BIGINT,
  PROCESS_INSTANCE_KEY BIGINT,
  ELEMENT_INSTANCE_KEY BIGINT,
  TENANT_ID VARCHAR(255),
  DUE_DATE TIMESTAMP(3) WITH TIME ZONE,
  FOLLOW_UP_DATE TIMESTAMP(3) WITH TIME ZONE,
  EXTERNAL_FORM_REFERENCE VARCHAR(255),
  PROCESS_DEFINITION_VERSION SMALLINT,
  CUSTOM_HEADERS TEXT,
  PRIORITY INTEGER,
  PARTITION_ID numeric,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT "USER_TASK_pkey" PRIMARY KEY (USER_TASK_KEY)
);

CREATE TABLE CANDIDATE_USER (
  USER_TASK_KEY BIGINT,
  CANDIDATE_USER VARCHAR(255),
  CONSTRAINT FK_CANDIDATE_USER_USER_TASK FOREIGN KEY (USER_TASK_KEY) REFERENCES USER_TASK(USER_TASK_KEY) ON DELETE CASCADE
);

CREATE TABLE CANDIDATE_GROUP (
  USER_TASK_KEY BIGINT,
  CANDIDATE_GROUP VARCHAR(255),
  CONSTRAINT FK_CANDIDATE_GROUP_USER_TASK FOREIGN KEY (USER_TASK_KEY) REFERENCES USER_TASK(USER_TASK_KEY) ON DELETE CASCADE
);

CREATE INDEX IDX_CANDIDATE_USER_USER_TASK_KEY ON CANDIDATE_USER(USER_TASK_KEY);

CREATE INDEX IDX_CANDIDATE_GROUP_USER_TASK_KEY ON CANDIDATE_GROUP(USER_TASK_KEY);

-- create_incident
CREATE TABLE INCIDENT (
  INCIDENT_KEY BIGINT NOT NULL,
  FLOW_NODE_INSTANCE_KEY BIGINT,
  FLOW_NODE_ID VARCHAR(255),
  PROCESS_INSTANCE_KEY BIGINT,
  PROCESS_DEFINITION_ID VARCHAR(255),
  PROCESS_DEFINITION_KEY BIGINT,
  ERROR_MESSAGE VARCHAR(4000),
  ERROR_TYPE VARCHAR(100),
  STATE VARCHAR(20),
  CREATION_DATE TIMESTAMP(3) WITH TIME ZONE,
  END_DATE TIMESTAMP(3) WITH TIME ZONE,
  TREE_PATH VARCHAR(4000),
  TENANT_ID VARCHAR(255),
  JOB_KEY BIGINT,
  SCOPE_KEY BIGINT,
  PARTITION_ID numeric,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT "INCIDENT_pkey" PRIMARY KEY (INCIDENT_KEY)
);

-- create_decision_definition_table
CREATE TABLE DECISION_DEFINITION (
  DECISION_DEFINITION_KEY BIGINT NOT NULL,
  DECISION_DEFINITION_ID VARCHAR(255),
  NAME VARCHAR(255),
  TENANT_ID VARCHAR(255),
  VERSION SMALLINT,
  DECISION_REQUIREMENTS_KEY BIGINT,
  DECISION_REQUIREMENTS_ID VARCHAR(255),
  CONSTRAINT "DECISION_DEFINITION_pkey" PRIMARY KEY (DECISION_DEFINITION_KEY)
);

CREATE INDEX IDX_DECISION_DEFINITION_ID ON DECISION_DEFINITION(DECISION_DEFINITION_ID);

-- create_decision_requirements_table
CREATE TABLE DECISION_REQUIREMENTS (
  DECISION_REQUIREMENTS_KEY BIGINT NOT NULL,
  DECISION_REQUIREMENTS_ID VARCHAR(255),
  NAME VARCHAR(255),
  TENANT_ID VARCHAR(255),
  VERSION SMALLINT,
  RESOURCE_NAME VARCHAR(4000),
  XML TEXT,
  CONSTRAINT "DECISION_REQUIREMENTS_pkey" PRIMARY KEY (DECISION_REQUIREMENTS_KEY)
);

CREATE INDEX IDX_DECISION_REQUIREMENTS_ID ON DECISION_REQUIREMENTS(DECISION_REQUIREMENTS_ID);

-- create_decision_instance_table
CREATE TABLE DECISION_INSTANCE (
  DECISION_INSTANCE_ID VARCHAR(255) NOT NULL,
  DECISION_INSTANCE_KEY BIGINT,
  PROCESS_INSTANCE_KEY BIGINT,
  PROCESS_DEFINITION_ID VARCHAR(255),
  PROCESS_DEFINITION_KEY BIGINT,
  DECISION_DEFINITION_ID VARCHAR(255),
  DECISION_DEFINITION_KEY BIGINT,
  DECISION_REQUIREMENTS_ID VARCHAR(255),
  DECISION_REQUIREMENTS_KEY BIGINT,
  FLOW_NODE_ID VARCHAR(255),
  FLOW_NODE_INSTANCE_KEY BIGINT,
  ROOT_DECISION_DEFINITION_KEY BIGINT,
  EVALUATION_DATE TIMESTAMP(3) WITH TIME ZONE,
  TYPE VARCHAR(255),
  STATE VARCHAR(255),
  RESULT VARCHAR(255),
  EVALUATION_FAILURE VARCHAR(255),
  TENANT_ID VARCHAR(255),
  PARTITION_ID numeric,
  HISTORY_CLEANUP_DATE TIMESTAMP(3) WITH TIME ZONE,
  CONSTRAINT "DECISION_INSTANCE_pkey" PRIMARY KEY (DECISION_INSTANCE_ID)
);

-- create_decision_instance_input_table
CREATE TABLE DECISION_INSTANCE_INPUT (
  DECISION_INSTANCE_ID VARCHAR(255) NOT NULL,
  INPUT_ID VARCHAR(255) NOT NULL,
  INPUT_NAME VARCHAR(255),
  INPUT_VALUE VARCHAR(4000),
  CONSTRAINT "DECISION_INSTANCE_INPUT_pkey" PRIMARY KEY (DECISION_INSTANCE_ID, INPUT_ID),
  CONSTRAINT FK_DEC_INST_INPUT_DEC_INST FOREIGN KEY (DECISION_INSTANCE_ID) REFERENCES DECISION_INSTANCE(DECISION_INSTANCE_ID) ON DELETE CASCADE
);

-- create_decision_instance_output_table
CREATE TABLE DECISION_INSTANCE_OUTPUT (
  DECISION_INSTANCE_ID VARCHAR(255) NOT NULL,
  OUTPUT_ID VARCHAR(255) NOT NULL,
  RULE_INDEX SMALLINT NOT NULL,
  OUTPUT_NAME VARCHAR(255),
  OUTPUT_VALUE VARCHAR(4000),
  RULE_ID VARCHAR(255),
  CONSTRAINT "DECISION_INSTANCE_OUTPUT_pkey" PRIMARY KEY (DECISION_INSTANCE_ID, OUTPUT_ID, RULE_INDEX),
  CONSTRAINT FK_DEC_INST_OUTPUT_DEC_INST FOREIGN KEY (DECISION_INSTANCE_ID) REFERENCES DECISION_INSTANCE(DECISION_INSTANCE_ID) ON DELETE CASCADE
);

-- create_user_table
CREATE TABLE USERS (
  USERNAME VARCHAR(255) NOT NULL,
  USER_KEY BIGINT,
  NAME VARCHAR(255),
  EMAIL VARCHAR(255),
  PASSWORD VARCHAR(255),
  CONSTRAINT "USERS_pkey" PRIMARY KEY (USERNAME)
);

-- create_form_table
CREATE TABLE FORM (
  FORM_KEY BIGINT NOT NULL,
  FORM_ID VARCHAR(255),
  TENANT_ID VARCHAR(255),
  SCHEMA TEXT,
  VERSION BIGINT,
  IS_DELETED BOOLEAN,
  CONSTRAINT "FORM_pkey" PRIMARY KEY (FORM_KEY)
);

CREATE INDEX IDX_FORM_ID ON FORM(FORM_ID);

CREATE INDEX IDX_FORM_TENANT_ID ON FORM(TENANT_ID);

-- create_mapping_table
CREATE TABLE MAPPINGS (
  MAPPING_KEY BIGINT NOT NULL,
  CLAIM_NAME VARCHAR(255),
  CLAIM_VALUE VARCHAR(255),
  NAME VARCHAR(255),
  CONSTRAINT "MAPPINGS_pkey" PRIMARY KEY (MAPPING_KEY)
);

-- create_tenant_table
CREATE TABLE TENANT (
  TENANT_ID VARCHAR(255) NOT NULL,
  TENANT_KEY BIGINT,
  NAME VARCHAR(255),
  DESCRIPTION VARCHAR(255),
  CONSTRAINT "TENANT_pkey" PRIMARY KEY (TENANT_ID)
);

-- tenant_members_table
CREATE TABLE TENANT_MEMBER (
  TENANT_ID VARCHAR(255) NOT NULL,
  ENTITY_ID VARCHAR(255) NOT NULL,
  ENTITY_TYPE VARCHAR(255),
  CONSTRAINT "TENANT_MEMBER_pkey" PRIMARY KEY (TENANT_ID, ENTITY_ID)
);

-- create_role_table
CREATE TABLE ROLES (
  ROLE_KEY BIGINT NOT NULL,
  NAME VARCHAR(255),
  CONSTRAINT "ROLES_pkey" PRIMARY KEY (ROLE_KEY)
);

-- role_members_table
CREATE TABLE ROLE_MEMBER (
  ROLE_KEY BIGINT NOT NULL,
  ENTITY_KEY BIGINT NOT NULL,
  ENTITY_TYPE VARCHAR(255),
  CONSTRAINT "ROLE_MEMBER_pkey" PRIMARY KEY (ROLE_KEY, ENTITY_KEY)
);

-- create_group_table
CREATE TABLE GROUPS (
  GROUP_KEY BIGINT NOT NULL,
  NAME VARCHAR(255),
  CONSTRAINT "GROUPS_pkey" PRIMARY KEY (GROUP_KEY)
);

-- group_members_table
CREATE TABLE GROUP_MEMBER (
  GROUP_KEY BIGINT NOT NULL,
  ENTITY_KEY BIGINT NOT NULL,
  ENTITY_TYPE VARCHAR(255),
  CONSTRAINT "GROUP_MEMBER_pkey" PRIMARY KEY (GROUP_KEY, ENTITY_KEY)
);

-- create_authorization_permission_table
CREATE TABLE AUTHORIZATIONS (
  AUTHORIZATION_KEY BIGINT,
  OWNER_ID VARCHAR(255),
  OWNER_TYPE VARCHAR(255),
  RESOURCE_TYPE VARCHAR(255),
  RESOURCE_ID VARCHAR(255),
  PERMISSION_TYPE VARCHAR(255)
);

CREATE INDEX IDX_AUTHORIZATIONS_AUTHORIZATION_KEY ON AUTHORIZATIONS(AUTHORIZATION_KEY);

CREATE INDEX IDX_AUTHORIZATIONS ON AUTHORIZATIONS(OWNER_ID, OWNER_TYPE, RESOURCE_TYPE);

